	subroutine tension1der3C2coeff (xx,px,cx,dimx,dim)

!
!   Calculating de Boor points of the first derivative of the third
!   order tension spline curve
!
!   INPUT:
!       xx    = rank-one double precision array of size dimx+4,
!               the increasing array of knots, assumed to fulfill
!               the conditions:
!               xx(1)=xx(2)=xx(3)=xx(4)<xx(5)<...<xx(i)<xx(i+1)<...
!               <xx(dimx)<xx(dimx+1)=xx(dimx+2)=xx(dimx+3)=
!               xx(dimx+4); xx(1),...,xx(dimx+4) is the extended
!               partition with all interior knots of multiplicity 1,
!               and the boundary points of multiplicity 4
!       px    = rank-one double precision array of size dimx+3,
!               the array of tension parameters, assumed to be
!               nonnegative: px(i)>=0 is the tension parameter on
!               the subinterval [xx(i),xx(i+1));
!               px(1),px(2),px(3),px(dimx+1),px(dimx+2) and
!               px(dimx+3) can be arbitrary
!       dimx  = integer variable, dimx+4 is the size of the array xx
!       dim   = integer variable, the extent of the first dimension
!               in the array cx, assumed to be positive
!
!   INPUT and OUTPUT:
!       cx    = rank-two double precision array of shape dim by
!               dim;
!               ON INPUT: the array of de Boor points of the
!               third order tension spline, associated with the
!               extended partition xx;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [xx(i),xx(i+3)], so cx(1:dim,1) can be
!               arbitrary
!               ON OUTPUT: the array of the de Boor points of the
!               first derivative of the third order tension spline;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [xx(i),xx(i+2)] (cx(1:dim,1) and cx(1:dim,2)
!               will not be used)
!

	use functions

	implicit none

	integer, intent(in) :: dimx,dim
	double precision, intent(in) :: xx(dimx+4),px(dimx+3)	
	double precision, intent(inout) :: cx(dim,dimx)


	double precision :: h(2),kc2ph(2)
	integer :: i

!
!	Calculating the de Boor points of the first derivative
!
	h=0.0d0
	kc2ph=0.0d0
	do i=dimx,3,-1
		h(1)=xx(i+1)-xx(i)
		kc2ph(1)=h(1)*KC2(px(i)*h(1)/2)/2
		cx(1:dim,i)=(cx(1:dim,i)-cx(1:dim,i-1))/(kc2ph(1)+kc2ph(2))
		h(2)=h(1)
		kc2ph(2)=kc2ph(1)
	enddo


	end subroutine tension1der3C2coeff