	subroutine tension1der4C2 (x,xx,px,ileft,cx,dim,spl)

!
!   Calculating the first derivative of the tension spline curve
!
!   INPUT:
!       x     = double precision variable, the point at which the
!               the first derivative of the spline curve is
!               evaluated
!       xx    = rank-one double precision array of size dimx+4,
!               the increasing array of knots, assumed to fulfill
!               the conditions:
!               xx(1)=xx(2)=xx(3)=xx(4)<xx(5)<...<xx(i)<xx(i+1)<...
!               <xx(dimx)<xx(dimx+1)=xx(dimx+2)=xx(dimx+3)=
!               xx(dimx+4); xx(1),...,xx(dimx+4) is the extended
!               partition with all interior knots of multiplicity 1,
!               and the boundary points of multiplicity 4
!       px    = rank-one double precision array of size dimx+3,
!               the array of tension parameters, assumed to be
!               nonnegative: px(i)>=0 is the tension parameter on
!               the subinterval [xx(i),xx(i+1));
!               px(1),px(2),px(3),px(dimx+1),px(dimx+2) and
!               px(dimx+3) can be arbitrary
!       ileft = integer variable, the index of the array xx such
!               that x is contained in the subinterval
!               [xx(ileft),xx(ileft+1)]
!       cx    = rank-two double precision array of shape dim by
!               dimx, the array of de Boor points of the tension
!               spline, associated with the extended partition xx;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [xx(i),xx(i+4)]
!       dim   = integer variable, the extent of the first dimension
!               in the array cx, assumed to be positive
!
!   AUXILIARY TERM:
!      dimx   = size(xx)-4
!
!   OUTPUT:
!       spl   = rank-one double precision array of size dim, the
!               output value of the first derivative of the tension
!               spline curve at the point x
!

	use functions

	implicit none

	integer, intent(in) :: ileft,dim
	double precision, intent(in) :: x,xx(*),px(*),cx(dim,*)	!xx(dimx+4),px(dimx+3),cx(dim,dimx)
	double precision, intent(out) :: spl(dim)

	double precision :: t(-2:3),p(-2:2),c(dim,-3:0) ! t(0)<=x<=t(1)
	double precision :: h(-2:2)
	integer :: i

!
!	Storing only relevant data
!
	t(-2:3)=xx(ileft-2:ileft+3)
	p(-2:2)=px(ileft-2:ileft+2)
	c(1:dim,-3:0)=cx(1:dim,ileft-3:ileft)
!
!	Calculating the length of the subintervals involved
!
	do i=-2,2
		h(i)=t(i+1)-t(i)
	enddo
!
!	Calculating the de Boor points of the first derivative
!
	do i=0,-2,-1
		c(1:dim,i)=(c(1:dim,i)-c(1:dim,i-1))/KC3(h(i:i+2),p(i:i+2))
	enddo
!
!   Calculating the value of the first derivative of the tension
!   spline curve as the linear combination of the third order
!   tension B-splines
!
	call tension3C2 (x,t,p,3,c(1:dim,-2:0),dim,spl)


	end subroutine tension1der4C2