	subroutine tension1der4C2coeff (xx,px,cx,dimx,dim)

!
!   Calculating all de Boor points of the first derivative of the
!   tension spline curve
!
!   INPUT:
!       xx    = rank-one double precision array of size dimx+4,
!               the increasing array of knots, assumed to fulfill
!               the conditions:
!               xx(1)=xx(2)=xx(3)=xx(4)<xx(5)<...<xx(i)<xx(i+1)<...
!               <xx(dimx)<xx(dimx+1)=xx(dimx+2)=xx(dimx+3)=
!               xx(dimx+4); xx(1),...,xx(dimx+4) is the extended
!               partition with all interior knots of multiplicity 1,
!               and the boundary points of multiplicity 4
!       px    = rank-one double precision array of size dimx+3,
!               the array of tension parameters, assumed to be
!               nonnegative: px(i)>=0 is the tension parameter on
!               the subinterval [xx(i),xx(i+1));
!               px(1),px(2),px(3),px(dimx+1),px(dimx+2) and
!               px(dimx+3) can be arbitrary
!       dimx  = integer variable, dimx+4 is the size of the array xx
!       dim   = integer variable, the extent of the first dimension
!               in the array cx, assumed to be positive
!
!   INPUT and OUTPUT:
!       cx    = rank-two double precision array of shape dim by
!               dimx;
!               ON INPUT: the array of de Boor points of the tension
!               spline, associated with the extended partition xx;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [xx(i),xx(i+4)]
!               ON OUTPUT: the array of the de Boor points of the
!               first derivative of the tension spline;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [xx(i),xx(i+3)] (cx(1:dim,1) will not be
!               used)
!

	use functions

	implicit none

	integer, intent(in) :: dimx,dim
	double precision, intent(in) :: xx(dimx+4),px(dimx+3)	
	double precision, intent(inout) :: cx(dim,dimx)

	double precision :: h(3)
	integer :: i

!
!	Calculating the de Boor points of the first derivative
!
	h=0.0d0
	do i=dimx,2,-1
		h(1)=xx(i+1)-xx(i)
		cx(1:dim,i)=(cx(1:dim,i)-cx(1:dim,i-1))/KC3(h,px(i:i+2))
		h(2:3)=h(1:2)
	enddo


	end subroutine tension1der4C2coeff