	subroutine tension4C2 (x,xx,px,ileft,cx,dim,spl)

!
!   Quasi-Oslo type algorithm for tension spline curves
!
!   INPUT:
!       x     = double precision variable, the point at which the
!               spline curve is evaluated
!       xx    = rank-one double precision array of size dimx+4,
!               the increasing array of knots, assumed to fulfill
!               the conditions:
!               xx(1)=xx(2)=xx(3)=xx(4)<xx(5)<...<xx(i)<xx(i+1)<...
!               <xx(dimx)<xx(dimx+1)=xx(dimx+2)=xx(dimx+3)=
!               xx(dimx+4); xx(1),...,xx(dimx+4) is the extended
!               partition with all interior knots of multiplicity 1,
!               and the boundary points of multiplicity 4
!       px    = rank-one double precision array of size dimx+3,
!               the array of tension parameters, assumed to be
!               nonnegative: px(i)>=0 is the tension parameter on
!               the subinterval [xx(i),xx(i+1));
!               px(1),px(2),px(3),px(dimx+1),px(dimx+2) and
!               px(dimx+3) can be arbitrary
!       ileft = integer variable, the index of the array xx such
!               that x is contained in the subinterval
!               [xx(ileft),xx(ileft+1)]
!       cx    = rank-two double precision array of shape dim by
!               dimx, the array of de Boor points associated with
!               the extended partition xx;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [xx(i),xx(i+4)]
!       dim   = integer variable, the extent of the first dimension
!               in the array cx, assumed to be positive
!
!   AUXILIARY TERM:
!      dimx   = size(xx)-4
!
!   OUTPUT:
!       spl   = rank-one double precision array of size dim, the
!               output value of the tension spline curve at the
!               point x
!

	use functions

	implicit none

	integer, intent(in) :: ileft,dim
	double precision, intent(in) :: x,xx(*),px(*),cx(dim,*)	
	double precision, intent(out) :: spl(dim)

	double precision :: t(-2:3),p(-2:2),c(dim,-3:0) ! t(0)<=x<=t(1)
	integer :: i,j
	double precision :: h(-2:2),cc(4,4),tspl(4),Bspl1,Bspl2,Bspl3,Bspl4,d(4)

!
!	Storing only relevant data
!
	t(-2:3)=xx(ileft-2:ileft+3)
	p(-2:2)=px(ileft-2:ileft+2)
	c(1:dim,-3:0)=cx(1:dim,ileft-3:ileft)
!
!	Calculating the length of the subintervals involved
!
	do i=-2,2
		h(i)=t(i+1)-t(i)
	enddo
!
!   Calculating nontrivial Chebyshev tension B-splines at the
!   point x
! 
	cc=0.0d0
	do i=1,4
		cc(i,i)=1.0d0
	enddo

	call tension4C1 (x,t(-1:2),p(-1:1),2,cc,4,tspl)
!
!   Calculating tension B-splines as the linear combination of the
!   Chebyshev tension B-splines
!
	Bspl1=gamma4d4(h(-2:0),p(-2:0))*tspl(1)
	call gamma4(h(-2:1),p(-2:1),d)
	Bspl2=d(2)*tspl(1)+d(3)*tspl(2)+d(4)*tspl(3)
	call gamma4(h(-1:2),p(-1:2),d)
	Bspl3=d(1)*tspl(2)+d(2)*tspl(3)+d(3)*tspl(4)
	Bspl4=gamma4d1(h(0:2),p(0:2))*tspl(4)
!
!	The final value of the spline curve
!
	spl=c(1:dim,-3)*Bspl1+c(1:dim,-2)*Bspl2+c(1:dim,-1)*Bspl3+c(1:dim,0)*Bspl4


	end subroutine tension4C2